@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2024 The pygit2 contributors
+ * Copyright 2010-2025 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -290,7 +290,7 @@ PyDoc_STRVAR(filter_register__doc__,
     "\n"
     "`priority` defaults to GIT_FILTER_DRIVER_PRIORITY which imitates a core\n"
     "Git filter driver that will be run last on checkout (smudge) and first \n"
-    "on checkin (clean).\n"
+    "on check-in (clean).\n"
     "\n"
     "Note that the filter registry is not thread safe. Any registering or\n"
     "deregistering of filters should be done outside of any possible usage\n"
