@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2014 The pygit2 contributors
+ * Copyright 2010-2025 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -31,191 +31,52 @@
 #include "error.h"
 #include "utils.h"
 #include "repository.h"
+#include "object.h"
 #include "oid.h"
 #include "tree.h"
 #include "diff.h"
 
 extern PyTypeObject TreeType;
-extern PyTypeObject TreeEntryType;
 extern PyTypeObject DiffType;
 extern PyTypeObject TreeIterType;
 extern PyTypeObject IndexType;
 
-void
-TreeEntry_dealloc(TreeEntry *self)
-{
-    git_tree_entry_free((git_tree_entry*)self->entry);
-    PyObject_Del(self);
-}
-
-
-PyDoc_STRVAR(TreeEntry_filemode__doc__, "Filemode.");
 
 PyObject *
-TreeEntry_filemode__get__(TreeEntry *self)
+treeentry_to_object(const git_tree_entry *entry, Repository *repo)
 {
-    return PyLong_FromLong(git_tree_entry_filemode(self->entry));
-}
-
-
-PyDoc_STRVAR(TreeEntry_name__doc__, "Name.");
-
-PyObject *
-TreeEntry_name__get__(TreeEntry *self)
-{
-    return to_path(git_tree_entry_name(self->entry));
-}
-
-
-PyDoc_STRVAR(TreeEntry_id__doc__, "Object id.");
-
-PyObject *
-TreeEntry_id__get__(TreeEntry *self)
-{
-    const git_oid *oid;
-
-    oid = git_tree_entry_id(self->entry);
-    return git_oid_to_python(oid);
-}
-
-PyDoc_STRVAR(TreeEntry_oid__doc__, "Object id.\n"
-    "This attribute is deprecated. Please use 'id'");
-
-PyObject *
-TreeEntry_oid__get__(TreeEntry *self)
-{
-    return TreeEntry_id__get__(self);
-}
-
-PyObject *
-TreeEntry_richcompare(PyObject *a, PyObject *b, int op)
-{
-    PyObject *res;
-    int cmp;
-
-    /* We only support comparing to another tree entry */
-    if (!PyObject_TypeCheck(b, &TreeEntryType)) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
+    if (repo == NULL) {
+        PyErr_SetString(PyExc_ValueError, "expected repository");
+        return NULL;
     }
 
-    cmp =git_tree_entry_cmp(((TreeEntry*)a)->entry, ((TreeEntry*)b)->entry);
-    switch (op) {
-        case Py_LT:
-            res = (cmp <= 0) ? Py_True: Py_False;
-            break;
-        case Py_LE:
-            res = (cmp < 0) ? Py_True: Py_False;
-            break;
-        case Py_EQ:
-            res = (cmp == 0) ? Py_True: Py_False;
-            break;
-        case Py_NE:
-            res = (cmp != 0) ? Py_True: Py_False;
-            break;
-        case Py_GT:
-            res = (cmp > 0) ? Py_True: Py_False;
-            break;
-        case Py_GE:
-            res = (cmp >= 0) ? Py_True: Py_False;
-            break;
-        default:
-            PyErr_Format(PyExc_RuntimeError, "Unexpected '%d' op", op);
-            return NULL;
-    }
-
-    Py_INCREF(res);
-    return res;
+    return wrap_object(NULL, repo, entry);
 }
 
-
-PyDoc_STRVAR(TreeEntry_hex__doc__, "Hex oid.");
-
-PyObject *
-TreeEntry_hex__get__(TreeEntry *self)
-{
-    return git_oid_to_py_str(git_tree_entry_id(self->entry));
-}
-
-
-PyGetSetDef TreeEntry_getseters[] = {
-    GETTER(TreeEntry, filemode),
-    GETTER(TreeEntry, name),
-    GETTER(TreeEntry, oid),
-    GETTER(TreeEntry, id),
-    GETTER(TreeEntry, hex),
-    {NULL}
-};
-
-
-PyDoc_STRVAR(TreeEntry__doc__, "TreeEntry objects.");
-
-PyTypeObject TreeEntryType = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    "_pygit2.TreeEntry",                       /* tp_name           */
-    sizeof(TreeEntry),                         /* tp_basicsize      */
-    0,                                         /* tp_itemsize       */
-    (destructor)TreeEntry_dealloc,             /* tp_dealloc        */
-    0,                                         /* tp_print          */
-    0,                                         /* tp_getattr        */
-    0,                                         /* tp_setattr        */
-    0,                                         /* tp_compare        */
-    0,                                         /* tp_repr           */
-    0,                                         /* tp_as_number      */
-    0,                                         /* tp_as_sequence    */
-    0,                                         /* tp_as_mapping     */
-    0,                                         /* tp_hash           */
-    0,                                         /* tp_call           */
-    0,                                         /* tp_str            */
-    0,                                         /* tp_getattro       */
-    0,                                         /* tp_setattro       */
-    0,                                         /* tp_as_buffer      */
-    Py_TPFLAGS_DEFAULT,                        /* tp_flags          */
-    TreeEntry__doc__,                          /* tp_doc            */
-    0,                                         /* tp_traverse       */
-    0,                                         /* tp_clear          */
-    (richcmpfunc)TreeEntry_richcompare,        /* tp_richcompare    */
-    0,                                         /* tp_weaklistoffset */
-    0,                                         /* tp_iter           */
-    0,                                         /* tp_iternext       */
-    0,                                         /* tp_methods        */
-    0,                                         /* tp_members        */
-    TreeEntry_getseters,                       /* tp_getset         */
-    0,                                         /* tp_base           */
-    0,                                         /* tp_dict           */
-    0,                                         /* tp_descr_get      */
-    0,                                         /* tp_descr_set      */
-    0,                                         /* tp_dictoffset     */
-    0,                                         /* tp_init           */
-    0,                                         /* tp_alloc          */
-    0,                                         /* tp_new            */
-};
-
 Py_ssize_t
 Tree_len(Tree *self)
 {
-    assert(self->tree);
+    if (Object__load((Object*)self) == NULL) { return -1; } // Lazy load
     return (Py_ssize_t)git_tree_entrycount(self->tree);
 }
 
 int
 Tree_contains(Tree *self, PyObject *py_name)
 {
-    int err;
-    git_tree_entry *entry;
-    char *name;
+    if (Object__load((Object*)self) == NULL) { return -1; } // Lazy load
 
-    name = py_path_to_c_str(py_name);
+    PyObject *tvalue;
+    char *name = pgit_borrow_fsdefault(py_name, &tvalue);
     if (name == NULL)
         return -1;
 
-    err = git_tree_entry_bypath(&entry, self->tree, name);
-    free(name);
+    git_tree_entry *entry;
+    int err = git_tree_entry_bypath(&entry, self->tree, name);
+    Py_DECREF(tvalue);
 
-    if (err == GIT_ENOTFOUND)
+    if (err == GIT_ENOTFOUND) {
         return 0;
-
-    if (err < 0) {
+    } else if (err < 0) {
         Error_set(err);
         return -1;
     }
@@ -225,20 +86,8 @@ Tree_contains(Tree *self, PyObject *py_name)
     return 1;
 }
 
-TreeEntry *
-wrap_tree_entry(const git_tree_entry *entry)
-{
-    TreeEntry *py_entry;
-
-    py_entry = PyObject_New(TreeEntry, &TreeEntryType);
-    if (py_entry)
-        py_entry->entry = entry;
-
-    return py_entry;
-}
-
 int
-Tree_fix_index(Tree *self, PyObject *py_index)
+Tree_fix_index(const git_tree *tree, PyObject *py_index)
 {
     long index;
     size_t len;
@@ -248,7 +97,7 @@ Tree_fix_index(Tree *self, PyObject *py_index)
     if (PyErr_Occurred())
         return -1;
 
-    len = git_tree_entrycount(self->tree);
+    len = git_tree_entrycount(tree);
     slen = (long)len;
     if (index >= slen) {
         PyErr_SetObject(PyExc_IndexError, py_index);
@@ -271,6 +120,8 @@ Tree_iter(Tree *self)
 {
     TreeIter *iter;
 
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
     iter = PyObject_New(TreeIter, &TreeIterType);
     if (iter) {
         Py_INCREF(self);
@@ -280,18 +131,18 @@ Tree_iter(Tree *self)
     return (PyObject*)iter;
 }
 
-TreeEntry *
-Tree_getitem_by_index(Tree *self, PyObject *py_index)
+PyObject*
+tree_getentry_by_index(const git_tree *tree, Repository *repo, PyObject *py_index)
 {
     int index;
     const git_tree_entry *entry_src;
     git_tree_entry *entry;
 
-    index = Tree_fix_index(self, py_index);
+    index = Tree_fix_index(tree, py_index);
     if (PyErr_Occurred())
         return NULL;
 
-    entry_src = git_tree_entry_byindex(self->tree, index);
+    entry_src = git_tree_entry_byindex(tree, index);
     if (!entry_src) {
         PyErr_SetObject(PyExc_IndexError, py_index);
         return NULL;
@@ -302,172 +153,238 @@ Tree_getitem_by_index(Tree *self, PyObject *py_index)
         return NULL;
     }
 
-    return wrap_tree_entry(entry);
+    return treeentry_to_object(entry, repo);
 }
 
-TreeEntry *
-Tree_getitem(Tree *self, PyObject *value)
+PyObject*
+tree_getentry_by_path(const git_tree *tree, Repository *repo, PyObject *py_path)
 {
-    char *path;
-    git_tree_entry *entry;
-    int err;
-
-    /* Case 1: integer */
-    if (PyLong_Check(value))
-        return Tree_getitem_by_index(self, value);
-
-    /* Case 2: byte or text string */
-    path = py_path_to_c_str(value);
-    if (path == NULL)
+    PyObject *tvalue;
+    char *path = pgit_borrow_fsdefault(py_path, &tvalue);
+    if (path == NULL) {
+        PyErr_SetString(PyExc_TypeError, "Value must be a path string");
         return NULL;
+    }
 
-    err = git_tree_entry_bypath(&entry, self->tree, path);
-    free(path);
+    git_tree_entry *entry;
+    int err = git_tree_entry_bypath(&entry, tree, path);
+    Py_DECREF(tvalue);
 
     if (err == GIT_ENOTFOUND) {
-        PyErr_SetObject(PyExc_KeyError, value);
+        PyErr_SetObject(PyExc_KeyError, py_path);
         return NULL;
     }
 
     if (err < 0)
-        return (TreeEntry*)Error_set(err);
+        return Error_set(err);
 
     /* git_tree_entry_dup is already done in git_tree_entry_bypath */
-    return wrap_tree_entry(entry);
+    return treeentry_to_object(entry, repo);
+}
+
+PyObject*
+Tree_subscript(Tree *self, PyObject *value)
+{
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
+    /* Case 1: integer */
+    if (PyLong_Check(value))
+        return tree_getentry_by_index(self->tree, self->repo, value);
+
+    /* Case 2: byte or text string */
+    return tree_getentry_by_path(self->tree, self->repo, value);
+}
+
+PyObject *
+Tree_divide(Tree *self, PyObject *value)
+{
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+    return tree_getentry_by_path(self->tree, self->repo, value);
 }
 
 
 PyDoc_STRVAR(Tree_diff_to_workdir__doc__,
-  "diff_to_workdir([flags, context_lines, interhunk_lines]) -> Diff\n"
+  "diff_to_workdir(flags: enums.DiffOption = enums.DiffOption.NORMAL, context_lines: int = 3, interhunk_lines: int = 0) -> Diff\n"
   "\n"
   "Show the changes between the :py:class:`~pygit2.Tree` and the workdir.\n"
   "\n"
-  "Arguments:\n"
+  "Parameters:\n"
   "\n"
-  "flag: a GIT_DIFF_* constant.\n"
+  "flags\n"
+  "    A combination of enums.DiffOption constants.\n"
   "\n"
-  "context_lines: the number of unchanged lines that define the boundary\n"
-  "   of a hunk (and to display before and after)\n"
+  "context_lines\n"
+  "    The number of unchanged lines that define the boundary of a hunk\n"
+  "    (and to display before and after).\n"
   "\n"
-  "interhunk_lines: the maximum number of unchanged lines between hunk\n"
-  "   boundaries before the hunks will be merged into a one.\n");
+  "interhunk_lines\n"
+  "    The maximum number of unchanged lines between hunk boundaries before\n"
+  "    the hunks will be merged into a one.\n");
 
 PyObject *
-Tree_diff_to_workdir(Tree *self, PyObject *args)
+Tree_diff_to_workdir(Tree *self, PyObject *args, PyObject *kwds)
 {
     git_diff_options opts = GIT_DIFF_OPTIONS_INIT;
     git_diff *diff;
-    Repository *py_repo;
     int err;
 
-    if (!PyArg_ParseTuple(args, "|IHH", &opts.flags, &opts.context_lines,
-                                        &opts.interhunk_lines))
+    char *keywords[] = {"flags", "context_lines", "interhunk_lines", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|IHH", keywords, &opts.flags,
+                                     &opts.context_lines, &opts.interhunk_lines))
         return NULL;
 
-    py_repo = self->repo;
-    err = git_diff_tree_to_workdir(&diff, py_repo->repo, self->tree, &opts);
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
+    err = git_diff_tree_to_workdir(&diff, self->repo->repo, self->tree, &opts);
     if (err < 0)
         return Error_set(err);
 
-    return wrap_diff(diff, py_repo);
+    return wrap_diff(diff, self->repo);
 }
 
 
 PyDoc_STRVAR(Tree_diff_to_index__doc__,
-  "diff_to_index(index, [flags, context_lines, interhunk_lines]) -> Diff\n"
+  "diff_to_index(index: Index, flags: enums.DiffOption = enums.DiffOption.NORMAL, context_lines: int = 3, interhunk_lines: int = 0) -> Diff\n"
   "\n"
   "Show the changes between the index and a given :py:class:`~pygit2.Tree`.\n"
   "\n"
-  "Arguments:\n"
+  "Parameters:\n"
   "\n"
-  "tree: the :py:class:`~pygit2.Tree` to diff.\n"
+  "index : :py:class:`~pygit2.Index`\n"
+  "    The index to diff.\n"
   "\n"
-  "flag: a GIT_DIFF_* constant.\n"
+  "flags\n"
+  "    A combination of enums.DiffOption constants.\n"
   "\n"
-  "context_lines: the number of unchanged lines that define the boundary\n"
-  "   of a hunk (and to display before and after)\n"
+  "context_lines\n"
+  "    The number of unchanged lines that define the boundary of a hunk\n"
+  "    (and to display before and after).\n"
   "\n"
-  "interhunk_lines: the maximum number of unchanged lines between hunk\n"
-  "   boundaries before the hunks will be merged into a one.\n");
+  "interhunk_lines\n"
+  "    The maximum number of unchanged lines between hunk boundaries before\n"
+  "    the hunks will be merged into a one.\n");
 
 PyObject *
 Tree_diff_to_index(Tree *self, PyObject *args, PyObject *kwds)
 {
     git_diff_options opts = GIT_DIFF_OPTIONS_INIT;
     git_diff *diff;
-    Repository *py_repo;
+    git_index *index;
+    char *buffer;
+    Py_ssize_t length;
+    PyObject *py_idx;
     int err;
 
-    Index *py_idx = NULL;
-
-    if (!PyArg_ParseTuple(args, "O!|IHH", &IndexType, &py_idx, &opts.flags,
+    if (!PyArg_ParseTuple(args, "O|IHH", &py_idx, &opts.flags,
                                         &opts.context_lines,
                                         &opts.interhunk_lines))
         return NULL;
 
-    py_repo = self->repo;
-    err = git_diff_tree_to_index(&diff, py_repo->repo, self->tree,
-                                 py_idx->index, &opts);
+    /* Check whether the first argument is an index.
+     * FIXME Uses duck typing. This would be easy and correct if we had
+     * _pygit2.Index. */
+    PyObject *pygit2_index = PyObject_GetAttrString(py_idx, "_index");
+    if (!pygit2_index) {
+        PyErr_SetString(PyExc_TypeError, "argument must be an Index");
+        return NULL;
+    }
+    Py_DECREF(pygit2_index);
+
+    /* Get git_index from cffi's pointer */
+    PyObject *py_idx_ptr = PyObject_GetAttrString(py_idx, "_pointer");
+    if (!py_idx_ptr)
+        return NULL;
+
+    /* Here we need to do the opposite conversion from the _pointer getters */
+    if (PyBytes_AsStringAndSize(py_idx_ptr, &buffer, &length))
+        goto error;
+
+    if (length != sizeof(git_index *)) {
+        PyErr_SetString(PyExc_TypeError, "passed value is not a pointer");
+        goto error;
+    }
+
+    index = *((git_index **) buffer); /* the "buffer" contains the pointer */
+
+    /* Call git_diff_tree_to_index */
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
+    err = git_diff_tree_to_index(&diff, self->repo->repo, self->tree, index, &opts);
+    Py_DECREF(py_idx_ptr);
+
     if (err < 0)
         return Error_set(err);
 
-    return wrap_diff(diff, py_repo);
+    return wrap_diff(diff, self->repo);
+
+error:
+    Py_DECREF(py_idx_ptr);
+    return NULL;
 }
 
 
 PyDoc_STRVAR(Tree_diff_to_tree__doc__,
-  "diff_to_tree([tree, flags, context_lines, interhunk_lines, swap]) -> Diff\n"
+  "diff_to_tree([tree: Tree, flags: enums.DiffOption = enums.DiffOption.NORMAL, context_lines: int = 3, interhunk_lines: int = 0, swap: bool = False]) -> Diff\n"
   "\n"
-  "Show the changes between two trees\n"
+  "Show the changes between two trees.\n"
   "\n"
-  "Arguments:\n"
+  "Parameters:\n"
   "\n"
-  "tree: the :py:class:`~pygit2.Tree` to diff. If no tree is given the empty\n"
-  "   tree will be used instead.\n"
+  "tree: :py:class:`~pygit2.Tree`\n"
+  "    The tree to diff. If no tree is given the empty tree will be used\n"
+  "    instead.\n"
   "\n"
-  "flag: a GIT_DIFF_* constant.\n"
+  "flags\n"
+  "    A combination of enums.DiffOption constants.\n"
   "\n"
-  "context_lines: the number of unchanged lines that define the boundary\n"
-  "   of a hunk (and to display before and after)\n"
+  "context_lines\n"
+  "    The number of unchanged lines that define the boundary of a hunk\n"
+  "    (and to display before and after).\n"
   "\n"
-  "interhunk_lines: the maximum number of unchanged lines between hunk\n"
-  "   boundaries before the hunks will be merged into a one.\n"
+  "interhunk_lines\n"
+  "    The maximum number of unchanged lines between hunk boundaries before\n"
+  "    the hunks will be merged into a one.\n"
   "\n"
-  "swap: instead of diffing a to b. Diff b to a.\n");
+  "swap\n"
+  "    Instead of diffing a to b. Diff b to a.\n");
 
 PyObject *
 Tree_diff_to_tree(Tree *self, PyObject *args, PyObject *kwds)
 {
     git_diff_options opts = GIT_DIFF_OPTIONS_INIT;
     git_diff *diff;
-    git_tree *from, *to, *tmp;
-    Repository *py_repo;
+    git_tree *from, *to = NULL, *tmp;
     int err, swap = 0;
-    char *keywords[] = {"obj", "flags", "context_lines", "interhunk_lines",
-                        "swap", NULL};
+    char *keywords[] = {"obj", "flags", "context_lines", "interhunk_lines", "swap", NULL};
 
-    Tree *py_tree = NULL;
+    Tree *other = NULL;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O!IHHi", keywords,
-                                     &TreeType, &py_tree, &opts.flags,
+                                     &TreeType, &other, &opts.flags,
                                      &opts.context_lines,
                                      &opts.interhunk_lines, &swap))
         return NULL;
 
-    py_repo = self->repo;
-    to = (py_tree == NULL) ? NULL : py_tree->tree;
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
     from = self->tree;
+
+    if (other) {
+        if (Object__load((Object*)other) == NULL) { return NULL; } // Lazy load
+        to = other->tree;
+    }
+
     if (swap > 0) {
         tmp = from;
         from = to;
         to = tmp;
     }
 
-    err = git_diff_tree_to_tree(&diff, py_repo->repo, from, to, &opts);
+    err = git_diff_tree_to_tree(&diff, self->repo->repo, from, to, &opts);
     if (err < 0)
         return Error_set(err);
 
-    return wrap_diff(diff, py_repo);
+    return wrap_diff(diff, self->repo);
 }
 
 
@@ -484,17 +401,58 @@ PySequenceMethods Tree_as_sequence = {
 
 PyMappingMethods Tree_as_mapping = {
     (lenfunc)Tree_len,            /* mp_length */
-    (binaryfunc)Tree_getitem,     /* mp_subscript */
+    (binaryfunc)Tree_subscript,   /* mp_subscript */
     0,                            /* mp_ass_subscript */
 };
 
 PyMethodDef Tree_methods[] = {
     METHOD(Tree, diff_to_tree, METH_VARARGS | METH_KEYWORDS),
-    METHOD(Tree, diff_to_workdir, METH_VARARGS),
+    METHOD(Tree, diff_to_workdir, METH_VARARGS | METH_KEYWORDS),
     METHOD(Tree, diff_to_index, METH_VARARGS | METH_KEYWORDS),
     {NULL}
 };
 
+/* Py2/3 compatible structure
+ * see https://py3c.readthedocs.io/en/latest/ext-types.html#pynumbermethods
+ */
+PyNumberMethods Tree_as_number = {
+    0,                          /* nb_add */
+    0,                          /* nb_subtract */
+    0,                          /* nb_multiply */
+    0,                          /* nb_remainder */
+    0,                          /* nb_divmod */
+    0,                          /* nb_power */
+    0,                          /* nb_negative */
+    0,                          /* nb_positive */
+    0,                          /* nb_absolute */
+    0,                          /* nb_bool (Py2: nb_nonzero) */
+    0,                          /* nb_invert */
+    0,                          /* nb_lshift */
+    0,                          /* nb_rshift */
+    0,                          /* nb_and */
+    0,                          /* nb_xor */
+    0,                          /* nb_or */
+    0,                          /* nb_int */
+    0,                          /* nb_reserved (Py2: nb_long) */
+    0,                          /* nb_float */
+    0,                          /* nb_inplace_add */
+    0,                          /* nb_inplace_subtract */
+    0,                          /* nb_inplace_multiply */
+    0,                          /* nb_inplace_remainder */
+    0,                          /* nb_inplace_power */
+    0,                          /* nb_inplace_lshift */
+    0,                          /* nb_inplace_rshift */
+    0,                          /* nb_inplace_and */
+    0,                          /* nb_inplace_xor */
+    0,                          /* nb_inplace_or */
+    0,                          /* nb_floor_divide */
+    (binaryfunc)Tree_divide,    /* nb_true_divide */
+    0,                          /* nb_inplace_floor_divide */
+    0,                          /* nb_inplace_true_divide */
+    0,                          /* nb_index */
+    0,                          /* nb_matrix_multiply */
+    0,                          /* nb_inplace_matrix_multiply */
+};
 
 PyDoc_STRVAR(Tree__doc__, "Tree objects.");
 
@@ -508,8 +466,8 @@ PyTypeObject TreeType = {
     0,                                         /* tp_getattr        */
     0,                                         /* tp_setattr        */
     0,                                         /* tp_compare        */
-    0,                                         /* tp_repr           */
-    0,                                         /* tp_as_number      */
+    (reprfunc)Object_repr,                     /* tp_repr           */
+    &Tree_as_number,                           /* tp_as_number      */
     &Tree_as_sequence,                         /* tp_as_sequence    */
     &Tree_as_mapping,                          /* tp_as_mapping     */
     0,                                         /* tp_hash           */
@@ -518,7 +476,7 @@ PyTypeObject TreeType = {
     0,                                         /* tp_getattro       */
     0,                                         /* tp_setattro       */
     0,                                         /* tp_as_buffer      */
-    Py_TPFLAGS_DEFAULT,                        /* tp_flags          */
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,  /* tp_flags */
     Tree__doc__,                               /* tp_doc            */
     0,                                         /* tp_traverse       */
     0,                                         /* tp_clear          */
@@ -547,7 +505,7 @@ TreeIter_dealloc(TreeIter *self)
     PyObject_Del(self);
 }
 
-TreeEntry *
+PyObject*
 TreeIter_iternext(TreeIter *self)
 {
     const git_tree_entry *entry_src;
@@ -563,7 +521,8 @@ TreeIter_iternext(TreeIter *self)
         PyErr_SetNone(PyExc_MemoryError);
         return NULL;
     }
-    return wrap_tree_entry(entry);
+
+    return treeentry_to_object(entry, self->owner->repo);
 }
 
 
